# encoding: utf-8

require 'fiddle'
require 'fiddle/import'

# Kernel32Lib module
# supports kernel32 WinAPI functions
module Kernel32Lib
  extend Fiddle::Importer
  dlload 'kernel32.dll'
  extern 'unsigned long GetLastError(void)'

  class Error < RuntimeError

  end

  ERROR_CODES = []
  ERROR_CODES[0]='The operation completed successfully.'
  ERROR_CODES[1]='Incorrect function.'
  ERROR_CODES[2]='The system cannot find the file specified.'
  ERROR_CODES[3]='The system cannot find the path specified.'
  ERROR_CODES[4]='The system cannot open the file.'
  ERROR_CODES[5]='Access is denied.'
  ERROR_CODES[6]='The handle is invalid.'
  ERROR_CODES[7]='The storage control blocks were destroyed.'
  ERROR_CODES[8]='Not enough storage is available to process this command.'
  ERROR_CODES[9]='The storage control block address is invalid.'
  ERROR_CODES[10]='The environment is incorrect.'
  ERROR_CODES[11]='An attempt was made to load a program with an incorrect format.'
  ERROR_CODES[12]='The access code is invalid.'
  ERROR_CODES[13]='The data is invalid.'
  ERROR_CODES[14]='Not enough storage is available to complete this operation.'
  ERROR_CODES[15]='The system cannot find the drive specified.'
  ERROR_CODES[16]='The directory cannot be removed.'
  ERROR_CODES[17]='The system cannot move the file to a different disk drive.'
  ERROR_CODES[18]='There are no more files.'
  ERROR_CODES[19]='The media is write protected.'
  ERROR_CODES[20]='The system cannot find the device specified.'
  ERROR_CODES[21]='The device is not ready.'
  ERROR_CODES[22]='The device does not recognize the command.'
  ERROR_CODES[23]='Data error (cyclic redundancy check).'
  ERROR_CODES[24]='The program issued a command but the command length is incorrect.'
  ERROR_CODES[25]='The drive cannot locate a specific area or track on the disk.'
  ERROR_CODES[26]='The specified disk or diskette cannot be accessed.'
  ERROR_CODES[27]='The drive cannot find the sector requested.'
  ERROR_CODES[28]='The printer is out of paper.'
  ERROR_CODES[29]='The system cannot write to the specified device.'
  ERROR_CODES[30]='The system cannot read from the specified device.'
  ERROR_CODES[31]='A device attached to the system is not functioning.'
  ERROR_CODES[32]='The process cannot access the file because it is being used by another process.'
  ERROR_CODES[33]='The process cannot access the file because another process has locked a portion of the file.'
  ERROR_CODES[34]='The wrong diskette is in the drive. Insert %2 (Volume Serial Number: %3) into drive %1.'
  ERROR_CODES[36]='Too many files opened for sharing.'
  ERROR_CODES[38]='Reached the end of the file.'
  ERROR_CODES[39]='The disk is full.'
  ERROR_CODES[50]='The request is not supported.'
  ERROR_CODES[51]='Windows cannot find the network path. Verify that the network path is correct and the destination computer is not busy or turned off. If Windows still cannot find the network path, contact your network administrator.'
  ERROR_CODES[52]='You were not connected because a duplicate name exists on the network. If joining a domain, go to System in Control Panel to change the computer name and try again. If joining a workgroup, choose another workgroup name.'
  ERROR_CODES[53]='The network path was not found.'
  ERROR_CODES[54]='The network is busy.'
  ERROR_CODES[55]='The specified network resource or device is no longer available.'
  ERROR_CODES[56]='The network BIOS command limit has been reached.'
  ERROR_CODES[57]='A network adapter hardware error occurred.'
  ERROR_CODES[58]='The specified server cannot perform the requested operation.'
  ERROR_CODES[59]='An unexpected network error occurred.'
  ERROR_CODES[60]='The remote adapter is not compatible.'
  ERROR_CODES[61]='The printer queue is full.'
  ERROR_CODES[62]='Space to store the file waiting to be printed is not available on the server.'
  ERROR_CODES[63]='Your file waiting to be printed was deleted.'
  ERROR_CODES[64]='The specified network name is no longer available.'
  ERROR_CODES[65]='Network access is denied.'
  ERROR_CODES[66]='The network resource type is not correct.'
  ERROR_CODES[67]='The network name cannot be found.'
  ERROR_CODES[68]='The name limit for the local computer network adapter card was exceeded.'
  ERROR_CODES[69]='The network BIOS session limit was exceeded.'
  ERROR_CODES[70]='The remote server has been paused or is in the process of being started.'
  ERROR_CODES[71]='No more connections can be made to this remote computer at this time because there are already as many connections as the computer can accept.'
  ERROR_CODES[72]='The specified printer or disk device has been paused.'
  ERROR_CODES[80]='The file exists.'
  ERROR_CODES[82]='The directory or file cannot be created.'
  ERROR_CODES[83]='Fail on INT 24.'
  ERROR_CODES[84]='Storage to process this request is not available.'
  ERROR_CODES[85]='The local device name is already in use.'
  ERROR_CODES[86]='The specified network password is not correct.'
  ERROR_CODES[87]='The parameter is incorrect.'
  ERROR_CODES[88]='A write fault occurred on the network.'
  ERROR_CODES[89]='The system cannot start another process at this time.'
  ERROR_CODES[100]='Cannot create another system semaphore.'
  ERROR_CODES[101]='The exclusive semaphore is owned by another process.'
  ERROR_CODES[102]='The semaphore is set and cannot be closed.'
  ERROR_CODES[103]='The semaphore cannot be set again.'
  ERROR_CODES[104]='Cannot request exclusive semaphores at interrupt time.'
  ERROR_CODES[105]='The previous ownership of this semaphore has ended.'
  ERROR_CODES[106]='Insert the diskette for drive %1.'
  ERROR_CODES[107]='The program stopped because an alternate diskette was not inserted.'
  ERROR_CODES[108]='The disk is in use or locked by another process.'
  ERROR_CODES[109]='The pipe has been ended.'
  ERROR_CODES[110]='The system cannot open the device or file specified.'
  ERROR_CODES[111]='The file name is too long.'
  ERROR_CODES[112]='There is not enough space on the disk.'
  ERROR_CODES[113]='No more internal file identifiers available.'
  ERROR_CODES[114]='The target internal file identifier is incorrect.'
  ERROR_CODES[117]='The IOCTL call made by the application program is not correct.'
  ERROR_CODES[118]='The verify-on-write switch parameter value is not correct.'
  ERROR_CODES[119]='The system does not support the command requested.'
  ERROR_CODES[120]='This function is not supported on this system.'
  ERROR_CODES[121]='The semaphore timeout period has expired.'
  ERROR_CODES[122]='The data area passed to a system call is too small.'
  ERROR_CODES[123]='The filename, directory name, or volume label syntax is incorrect.'
  ERROR_CODES[124]='The system call level is not correct.'
  ERROR_CODES[125]='The disk has no volume label.'
  ERROR_CODES[126]='The specified module could not be found.'
  ERROR_CODES[127]='The specified procedure could not be found.'
  ERROR_CODES[128]='There are no child processes to wait for.'
  ERROR_CODES[129]='The %1 application cannot be run in Win32 mode.'
  ERROR_CODES[130]='Attempt to use a file handle to an open disk partition for an operation other than raw disk I/O.'
  ERROR_CODES[131]='An attempt was made to move the file pointer before the beginning of the file.'
  ERROR_CODES[132]='The file pointer cannot be set on the specified device or file.'
  ERROR_CODES[133]='A JOIN or SUBST command cannot be used for a drive that contains previously joined drives.'
  ERROR_CODES[134]='An attempt was made to use a JOIN or SUBST command on a drive that has already been joined.'
  ERROR_CODES[135]='An attempt was made to use a JOIN or SUBST command on a drive that has already been substituted.'
  ERROR_CODES[136]='The system tried to delete the JOIN of a drive that is not joined.'
  ERROR_CODES[137]='The system tried to delete the substitution of a drive that is not substituted.'
  ERROR_CODES[138]='The system tried to join a drive to a directory on a joined drive.'
  ERROR_CODES[139]='The system tried to substitute a drive to a directory on a substituted drive.'
  ERROR_CODES[140]='The system tried to join a drive to a directory on a substituted drive.'
  ERROR_CODES[141]='The system tried to SUBST a drive to a directory on a joined drive.'
  ERROR_CODES[142]='The system cannot perform a JOIN or SUBST at this time.'
  ERROR_CODES[143]='The system cannot join or substitute a drive to or for a directory on the same drive.'
  ERROR_CODES[144]='The directory is not a subdirectory of the root directory.'
  ERROR_CODES[145]='The directory is not empty.'
  ERROR_CODES[146]='The path specified is being used in a substitute.'
  ERROR_CODES[147]='Not enough resources are available to process this command.'
  ERROR_CODES[148]='The path specified cannot be used at this time.'
  ERROR_CODES[149]='An attempt was made to join or substitute a drive for which a directory on the drive is the target of a previous substitute.'
  ERROR_CODES[150]='System trace information was not specified in your CONFIG.SYS file, or tracing is disallowed.'
  ERROR_CODES[151]='The number of specified semaphore events for DosMuxSemWait is not correct.'
  ERROR_CODES[152]='DosMuxSemWait did not execute; too many semaphores are already set.'
  ERROR_CODES[153]='The DosMuxSemWait list is not correct.'
  ERROR_CODES[154]='The volume label you entered exceeds the label character limit of the target file system.'
  ERROR_CODES[155]='Cannot create another thread.'
  ERROR_CODES[156]='The recipient process has refused the signal.'
  ERROR_CODES[157]='The segment is already discarded and cannot be locked.'
  ERROR_CODES[158]='The segment is already unlocked.'
  ERROR_CODES[159]='The address for the thread ID is not correct.'
  ERROR_CODES[160]='One or more arguments are not correct.'
  ERROR_CODES[161]='The specified path is invalid.'
  ERROR_CODES[162]='A signal is already pending.'
  ERROR_CODES[164]='No more threads can be created in the system.'
  ERROR_CODES[167]='Unable to lock a region of a file.'
  ERROR_CODES[170]='The requested resource is in use.'
  ERROR_CODES[171]="Device's command support detection is in progress."
  ERROR_CODES[173]='A lock request was not outstanding for the supplied cancel region.'
  ERROR_CODES[174]='The file system does not support atomic changes to the lock type.'
  ERROR_CODES[180]='The system detected a segment number that was not correct.'
  ERROR_CODES[182]='The operating system cannot run %1.'
  ERROR_CODES[183]='Cannot create a file when that file already exists.'
  ERROR_CODES[186]='The flag passed is not correct.'
  ERROR_CODES[187]='The specified system semaphore name was not found.'
  ERROR_CODES[188]='The operating system cannot run %1.'
  ERROR_CODES[189]='The operating system cannot run %1.'
  ERROR_CODES[190]='The operating system cannot run %1.'
  ERROR_CODES[191]='Cannot run %1 in Win32 mode.'
  ERROR_CODES[192]='The operating system cannot run %1.'
  ERROR_CODES[193]='%1 is not a valid Win32 application.'
  ERROR_CODES[194]='The operating system cannot run %1.'
  ERROR_CODES[195]='The operating system cannot run %1.'
  ERROR_CODES[196]='The operating system cannot run this application program.'
  ERROR_CODES[197]='The operating system is not presently configured to run this application.'
  ERROR_CODES[198]='The operating system cannot run %1.'
  ERROR_CODES[199]='The operating system cannot run this application program.'
  ERROR_CODES[200]='The code segment cannot be greater than or equal to 64K.'
  ERROR_CODES[201]='The operating system cannot run %1.'
  ERROR_CODES[202]='The operating system cannot run %1.'
  ERROR_CODES[203]='The system could not find the environment option that was entered.'
  ERROR_CODES[205]='No process in the command subtree has a signal handler.'
  ERROR_CODES[206]='The filename or extension is too long.'
  ERROR_CODES[207]='The ring 2 stack is in use.'
  ERROR_CODES[208]='The global filename characters, * or ?, are entered incorrectly or too many global filename characters are specified.'
  ERROR_CODES[209]='The signal being posted is not correct.'
  ERROR_CODES[210]='The signal handler cannot be set.'
  ERROR_CODES[212]='The segment is locked and cannot be reallocated.'
  ERROR_CODES[214]='Too many dynamic-link modules are attached to this program or dynamic-link module.'
  ERROR_CODES[215]='Cannot nest calls to LoadModule.'
  ERROR_CODES[216]="This version of %1 is not compatible with the version of Windows you're running. Check your computer's system information and then contact the software publisher."
  ERROR_CODES[217]='The image file %1 is signed, unable to modify.'
  ERROR_CODES[218]='The image file %1 is strong signed, unable to modify.'
  ERROR_CODES[220]='This file is checked out or locked for editing by another user.'
  ERROR_CODES[221]='The file must be checked out before saving changes.'
  ERROR_CODES[222]='The file type being saved or retrieved has been blocked.'
  ERROR_CODES[223]='The file size exceeds the limit allowed and cannot be saved.'
  ERROR_CODES[224]='Access Denied. Before opening files in this location, you must first add the web site to your trusted sites list, browse to the web site, and select the option to login automatically.'
  ERROR_CODES[225]='Operation did not complete successfully because the file contains a virus or potentially unwanted software.'
  ERROR_CODES[226]='This file contains a virus or potentially unwanted software and cannot be opened. Due to the nature of this virus or potentially unwanted software, the file has been removed from this location.'
  ERROR_CODES[229]='The pipe is local.'
  ERROR_CODES[230]='The pipe state is invalid.'
  ERROR_CODES[231]='All pipe instances are busy.'
  ERROR_CODES[232]='The pipe is being closed.'
  ERROR_CODES[233]='No process is on the other end of the pipe.'
  ERROR_CODES[234]='More data is available.'
  ERROR_CODES[240]='The session was canceled.'
  ERROR_CODES[254]='The specified extended attribute name was invalid.'
  ERROR_CODES[255]='The extended attributes are inconsistent.'
  ERROR_CODES[258]='The wait operation timed out.'
  ERROR_CODES[259]='No more data is available.'
  ERROR_CODES[266]='The copy functions cannot be used.'
  ERROR_CODES[267]='The directory name is invalid.'
  ERROR_CODES[275]='The extended attributes did not fit in the buffer.'
  ERROR_CODES[276]='The extended attribute file on the mounted file system is corrupt.'
  ERROR_CODES[277]='The extended attribute table file is full.'
  ERROR_CODES[278]='The specified extended attribute handle is invalid.'
  ERROR_CODES[282]='The mounted file system does not support extended attributes.'
  ERROR_CODES[288]='Attempt to release mutex not owned by caller.'
  ERROR_CODES[298]='Too many posts were made to a semaphore.'
  ERROR_CODES[299]='Only part of a ReadProcessMemory or WriteProcessMemory request was completed.'
  ERROR_CODES[300]='The oplock request is denied.'
  ERROR_CODES[301]='An invalid oplock acknowledgment was received by the system.'
  ERROR_CODES[302]='The volume is too fragmented to complete this operation.'
  ERROR_CODES[303]='The file cannot be opened because it is in the process of being deleted.'
  ERROR_CODES[304]='Short name settings may not be changed on this volume due to the global registry setting.'
  ERROR_CODES[305]='Short names are not enabled on this volume.'
  ERROR_CODES[306]='The security stream for the given volume is in an inconsistent state. Please run CHKDSK on the volume.'
  ERROR_CODES[307]='A requested file lock operation cannot be processed due to an invalid byte range.'
  ERROR_CODES[308]='The subsystem needed to support the image type is not present.'
  ERROR_CODES[309]='The specified file already has a notification GUID associated with it.'
  ERROR_CODES[310]='An invalid exception handler routine has been detected.'
  ERROR_CODES[311]='Duplicate privileges were specified for the token.'
  ERROR_CODES[312]='No ranges for the specified operation were able to be processed.'
  ERROR_CODES[313]='Operation is not allowed on a file system internal file.'
  ERROR_CODES[314]='The physical resources of this disk have been exhausted.'
  ERROR_CODES[315]='The token representing the data is invalid.'
  ERROR_CODES[316]='The device does not support the command feature.'
  ERROR_CODES[317]='The system cannot find message text for message number 0x%1 in the message file for %2.'
  ERROR_CODES[318]='The scope specified was not found.'
  ERROR_CODES[319]='The Central Access Policy specified is not defined on the target machine.'
  ERROR_CODES[320]='The Central Access Policy obtained from Active Directory is invalid.'
  ERROR_CODES[321]='The device is unreachable.'
  ERROR_CODES[322]='The target device has insufficient resources to complete the operation.'
  ERROR_CODES[323]='A data integrity checksum error occurred. Data in the file stream is corrupt.'
  ERROR_CODES[324]='An attempt was made to modify both a KERNEL and normal Extended Attribute (EA) in the same operation.'
  ERROR_CODES[326]='Device does not support file-level TRIM.'
  ERROR_CODES[327]="The command specified a data offset that does not align to the device's granularity/alignment."
  ERROR_CODES[328]='The command specified an invalid field in its parameter list.'
  ERROR_CODES[329]='An operation is currently in progress with the device.'
  ERROR_CODES[330]='An attempt was made to send down the command via an invalid path to the target device.'
  ERROR_CODES[331]='The command specified a number of descriptors that exceeded the maximum supported by the device.'
  ERROR_CODES[332]='Scrub is disabled on the specified file.'
  ERROR_CODES[333]='The storage device does not provide redundancy.'
  ERROR_CODES[334]='An operation is not supported on a resident file.'
  ERROR_CODES[335]='An operation is not supported on a compressed file.'
  ERROR_CODES[336]='An operation is not supported on a directory.'
  ERROR_CODES[337]='The specified copy of the requested data could not be read.'
  ERROR_CODES[350]='No action was taken as a system reboot is required.'
  ERROR_CODES[351]='The shutdown operation failed.'
  ERROR_CODES[352]='The restart operation failed.'
  ERROR_CODES[353]='The maximum number of sessions has been reached.'
  ERROR_CODES[400]='The thread is already in background processing mode.'
  ERROR_CODES[401]='The thread is not in background processing mode.'
  ERROR_CODES[402]='The process is already in background processing mode.'
  ERROR_CODES[403]='The process is not in background processing mode.'
  ERROR_CODES[487]='Attempt to access invalid address.'

  # GetLastError
  def self.get_last_error
    Kernel32Lib.GetLastError(0)
  end

  def self.translate_error(error_code)
    (ERROR_CODES[error_code] || "Error message by code #{error_code} not found.")
  end

  def self.get_last_error_message
    translate_error(get_last_error)
  end
end
